==> 0.0.24.tar.gz: west-sound-hall/src/app/calendar/page.tsx <==
import * as ical from "ical";
import dayjs from "dayjs";
import React from 'react';
import { CalendarComponent } from "./calendar";
import { Container } from "@/components/Container";

const icalAddr = process.env.CALENDAR_ADDR || '';

export type Calendar = Record<number, MonthEvents>;
type MonthEvents = Record<number, DayEvents>;
type DayEvents = Record<number, Event[]>;

let calendar: Calendar;
let upcomingEvents: Event[] = [];
let calendarLastRefresh: dayjs.Dayjs;

export async function getCalendar(): Promise<Calendar> {
    const now = dayjs();
    if (!calendar || !calendarLastRefresh || now.diff(calendarLastRefresh, 'hours') >= 1) {
        await loadCalendar();
        calendarLastRefresh = now;
    }

    return calendar;
}

export async function getUpcomingEvents(): Promise<Event[]> {
    await getCalendar();
    return upcomingEvents;
}

/** Event must be a "plain object" so that it can be passed to the Calendar client component. */
export interface Event {
    id: string;
    name: string;
    allDay: boolean;
    start: Date;
    end: Date;
}

async function loadCalendar(): Promise<void> {
    if (icalAddr === '') {
        return;
    }

    const icalContents = await (await fetch(icalAddr)).text();
    const events = ical.parseICS(icalContents);
    const thisMonth = dayjs().startOf('month');
    const yesterday = dayjs().startOf('day').subtract(1, 'day');
    const upcoming: Event[] = []

    const cal: Calendar = {};
    for (const id in events) {
        if (!events.hasOwnProperty(id)) {
            continue;
        }

        const event = events[id];
        const start = dayjs(event.start);
        const end = dayjs(event.end);
        const allDay = start.hour() === 0 && start.minute() === 0 && end.diff(start, 'hours') === 24;
        const privateEvent = event.description?.toLowerCase().includes("private");

        const converted = {
            id: event.uid || "",
            name: privateEvent ? "Private event" : (event.summary || ""),
            allDay: allDay,
            start: start.toDate(),
            end: end.toDate(),
        }

        // Don't spend any more time on events that aren't going to be displayed.
        if (start.isBefore(thisMonth)) {
            continue;
        }

        if (upcoming.length < 4 && start.isAfter(yesterday) && !privateEvent) {
            upcoming.push(converted);
        }

        const year = start.year();
        const month = start.month();
        const day = start.date();

        if (!(year in cal)) {
            cal[year] = {};
        }
        const yearMap = cal[year];

        if (!(month in yearMap)) {
            yearMap[month] = {};
        }
        const monthMap = yearMap[month];

        if (day in monthMap) {
            monthMap[day].push(converted);
        } else {
            monthMap[day] = [converted];
        }
    }

    calendar = cal;
    upcomingEvents = upcoming;
}

export default async function Calendar() {
    const calendar = await getCalendar();

    return (
        <Container className="mt-16 sm:mt-32">
            <header className="max-w-2xl">
                <h1 className="text-4xl font-bold tracking-tight text-zinc-800 sm:text-5xl">
                    Hall Calendar
                </h1>
            </header>
            <CalendarComponent calendar={calendar} />
        </Container>
    );
}

==> 0.0.29.tar.gz: west-sound-hall/src/app/calendar/page.tsx <==
import * as ical from "ical";
import dayjs from "dayjs";
import React from 'react';
import { CalendarComponent } from "./calendar";
import { Container } from "@/components/Container";
import { Metadata } from "next";

const icalAddr = process.env.CALENDAR_ADDR || '';

export type Calendar = Record<number, MonthEvents>;
type MonthEvents = Record<number, DayEvents>;
type DayEvents = Record<number, Event[]>;

let calendar: Calendar;
let upcomingEvents: Event[] = [];
let calendarLastRefresh: dayjs.Dayjs;

export async function getCalendar(): Promise<Calendar> {
    const now = dayjs();
    if (!calendar || !calendarLastRefresh || now.diff(calendarLastRefresh, 'hours') >= 1) {
        await loadCalendar();
        calendarLastRefresh = now;
    }

    return calendar;
}

export async function getUpcomingEvents(): Promise<Event[]> {
    await getCalendar();
    return upcomingEvents;
}

/** Event must be a "plain object" so that it can be passed to the Calendar client component. */
export interface Event {
    id: string;
    name: string;
    allDay: boolean;
    start: Date;
    end: Date;
}

async function loadCalendar(): Promise<void> {
    if (icalAddr === '') {
        return;
    }

    console.log("Refreshing iCal from Google Calendar")

    // For some reason Google Calendar is sending different content based on who is requesting it.
    const headers = new Headers();
    headers.set('User-Agent', 'curl/7.54.1');

    const icalContents = await (await fetch(icalAddr, { headers })).text();
    const events = ical.parseICS(icalContents);
    const thisMonth = dayjs().startOf('month');
    const yesterday = dayjs().startOf('day').subtract(1, 'day');
    const upcoming: Event[] = []

    const cal: Calendar = {};
    for (const id in events) {
        if (!events.hasOwnProperty(id)) {
            continue;
        }

        const event = events[id];
        const start = dayjs(event.start);
        const end = dayjs(event.end);
        const allDay = start.hour() === 0 && start.minute() === 0 && end.diff(start, 'hours') === 24;
        const privateEvent = event.description?.toLowerCase().includes("private");

        const converted = {
            id: event.uid || "",
            name: privateEvent ? "Private event" : (event.summary || ""),
            allDay: allDay,
            start: start.toDate(),
            end: end.toDate(),
        }

        // Don't spend any more time on events that aren't going to be displayed.
        if (start.isBefore(thisMonth)) {
            continue;
        }

        if (upcoming.length < 4 && start.isAfter(yesterday) && !privateEvent) {
            upcoming.push(converted);
        }

        const year = start.year();
        const month = start.month();
        const day = start.date();

        if (!(year in cal)) {
            cal[year] = {};
        }
        const yearMap = cal[year];

        if (!(month in yearMap)) {
            yearMap[month] = {};
        }
        const monthMap = yearMap[month];

        if (day in monthMap) {
            monthMap[day].push(converted);
        } else {
            monthMap[day] = [converted];
        }
    }

    calendar = cal;
    upcomingEvents = upcoming;
}

export const metadata: Metadata = {
    title: 'Events Calendar',
    description:
        'Upcoming events and rentals ast the West Sound Hall.',
}

export default async function Calendar() {
    const calendar = await getCalendar();

    return (
        <Container className="mt-16 sm:mt-32">
            <header className="max-w-2xl">
                <h1 className="text-4xl font-bold tracking-tight text-zinc-800 sm:text-5xl">
                    West Sound Hall Calendar
                </h1>
            </header>
            <CalendarComponent calendar={calendar} />
        </Container>
    );
}
